import 'dart:async';
import 'dart:convert';

import 'package:bea_flutter_package/bea_services/dtos/bea_models/base_models/error/bea_error.dart';
import 'package:bea_flutter_package/bea_services/dtos/bea_models/base_models/mobile/get_current_mobile_version_input.dart';
import 'package:bea_flutter_package/bea_services/dtos/bea_models/base_models/mobile/get_current_mobile_version_output.dart';
import 'package:bea_flutter_package/bea_services/dtos/flutter_models/input/generic_server_input.dart';
import 'package:bea_flutter_package/bea_services/dtos/flutter_models/output/bea_session.dart';
import 'package:bea_flutter_package/bea_services/utils/url_utils.dart';
import 'package:dio/dio.dart';
import 'package:f_logs/model/flog/flog.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
//import 'package:http/http.dart';

import '../dtos/flutter_models/output/generic_server_response.dart';
import 'bea_service_manager.dart';

class BeaMobileManager{
  final BeaServiceManager? bsm;

  String _baseUrl = "";
  FlutterSecureStorage? storage;

  BeaMobileManager(this.bsm){
    try{
      if(bsm == null){
        throw new Exception("BeaServiceManager not initialized");
      }

      _baseUrl = bsm!.getBaseUrl();
      storage = bsm!.getBeaStorage();
    }on Exception catch (e){
      print('Initialization ${this.runtimeType.toString()} error');
      FLog.error(
          className: this.runtimeType.toString(),
          methodName: "Costructor",
          text: 'Initialization ${this.runtimeType.toString()} error - $e');
      throw e;
    }
  }

  Future<GenericServerResponse<GetCurrentMobileVersionOutput>> getCurrentVersion(GenericServerInput<GetCurrentMobileVersionInput>? input) async {
    GenericServerResponse<GetCurrentMobileVersionOutput> genericServerResponse = new GenericServerResponse<GetCurrentMobileVersionOutput>();
    try{

      if(input == null || input.input == null){
        throw new Exception("Input not found.");
      }

      String url = UrlUtils.getUrl(_baseUrl, UrlUtils.URL_GET_MOBILE_VERSION);

      String jsonInput = jsonEncode(input.input!.toJson());

      Response response = await bsm!.genericBeaHttpPost(url, jsonInput);
      int? statusCode = response.statusCode;

      switch(statusCode){
        case 200:
          genericServerResponse.data = GetCurrentMobileVersionOutput.fromJson(response.data['result']);
          break;
        case 500:
          var res = response.data['error'];
          BeaError? error = res == null ? null : BeaError.fromJson(res);
          if (error != null) {
            genericServerResponse.beaError = error;
          } else {
            genericServerResponse.beaError = new BeaError("Unknown Error", 0, "");
          }
          break;
        case 401:
          genericServerResponse.beaSession = new BeaSession(true);
          break;
      }
    }on Exception catch (e){
      print('Error to get current mobile version BEA');
      FLog.error(
          className: this.runtimeType.toString(),
          methodName: "getCurrentVersion",
          text: 'Error to get current mobile version BEA - $e');
      throw e;
    }

    return genericServerResponse;
  }
}